@@ -58,14 +58,22 @@ class DAppTonClient(TonClient):
 
         return address_infos
 
-    def deploy_wallet(self, wallet: WalletContract):
+    def seqno(self, addr: str):
+        if self.get_address_information(addr).seqno:
+            return int(self.get_address_information(addr).seqno)
+
+        return 0
+
+    def deploy_wallet(self, wallet: WalletContract, wait_for_result=False):
+        timeout = 20 if wait_for_result else 0
         query = wallet.create_init_external_message()
         base64_boc = bytes_to_b64str(query["message"].to_boc(False))
 
-        return self._run(self.provider.broadcast([BroadcastQuery(boc=base64_boc, timeout=0)]))
+        return self._run(self.provider.broadcast([BroadcastQuery(boc=base64_boc, timeout=timeout)]))
 
     def transfer(self, from_wallet: WalletContract, to_addr: str, amount: TonCurrencyEnum.ton, payload=None,
-                 send_mode=SendModeEnum.ignore_errors | SendModeEnum.pay_gas_separately) -> bool:
+                 send_mode=SendModeEnum.ignore_errors | SendModeEnum.pay_gas_separately, wait_for_result=False) -> bool:
+        timeout = 10 if wait_for_result else 0
         seqno = self.seqno(from_wallet.address.to_string())
         query = from_wallet.create_transfer_message(
             to_addr=Address(to_addr).to_string(False, False, False),
@@ -76,13 +84,7 @@ class DAppTonClient(TonClient):
         )
         msg_boc = query["message"].to_boc(False)
         base64_boc = bytes_to_b64str(msg_boc)
-        return self._run(self.provider.broadcast([BroadcastQuery(boc=base64_boc, timeout=0)]))
-
-    def seqno(self, addr: str):
-        if self.get_address_information(addr).seqno:
-            return int(self.get_address_information(addr).seqno)
-
-        return 0
+        return self._run(self.provider.broadcast([BroadcastQuery(boc=base64_boc, timeout=timeout)]))
 
     def _run(self, to_run, *, single_query=True):
         try:
