@@ -1,3 +1,4 @@
+from hashlib import new
 import os
 from collections import OrderedDict
 import inquirer
@@ -42,6 +43,7 @@ class KeystoreSet(BaseSet):
         ]
         verbose = self._prompt(questions)["verbose"]
 
+        self._start_loading()
         field_names = ['Name', 'Version', 'WC', 'Address', 'Comment']
         if verbose:
             field_names += ['State', 'Balance']
@@ -62,7 +64,7 @@ class KeystoreSet(BaseSet):
             for wallet in wallets:
                 table.add_row([wallet.name, wallet.version, wallet.workchain,
                                wallet.address_to_show, wallet.comment])
-
+        self._stop_loading()
         echo_success(table, only_msg=True)
 
     def _handle_transfer(self):
@@ -93,6 +95,8 @@ class KeystoreSet(BaseSet):
                 "destroy_if_zero", message='Destroy if zero?', default=False),
             inquirer.Confirm(
                 "transfer_all", message='Transfer all?', default=False),
+            inquirer.Confirm(
+                "wait_for_result", message="Wait until transaction will be completed?", default=True),
         ]
         ans = self._prompt(questions)
 
@@ -100,6 +104,7 @@ class KeystoreSet(BaseSet):
         message = ans["message"]
         destroy_if_zero = ans["destroy_if_zero"]
         transfer_all = ans["transfer_all"]
+        wait_for_result = ans["wait_for_result"]
         send_mode = SendModeEnum.ignore_errors | SendModeEnum.pay_gas_separately
         if destroy_if_zero:
             send_mode |= SendModeEnum.destroy_account_if_zero
@@ -110,45 +115,119 @@ class KeystoreSet(BaseSet):
                                                                      record.workchain)
 
         result = self.ctx.ton_client.transfer(
-            wallet, contact.address, amount, message, send_mode)
+            wallet, contact.address, amount, message, send_mode, wait_for_result)
 
-        echo_success(result)
+        if wait_for_result:
+            self._parse_broadcast_result(result)
+        else:
+            echo_success()
 
     def _handle_create_wallet(self):
         questions = [
             inquirer.List(
-                "version", message='Wallet version', choices=[e.value for e in WalletVersionEnum], carousel=True),
+                "single", message="Choose option", choices=["Single", "Batch"],
+                carousel=True),
+            inquirer.List(
+                "version", message='Wallet version', choices=[e.value for e in WalletVersionEnum],
+                carousel=True, default=WalletVersionEnum.v4r2),
             inquirer.Text(
-                "workchain", message='Workchain'),
-            inquirer.Text("name", message='Wallet name'),
-            inquirer.Text(
-                "comment", message='Wallet description (leave blank to skip)'),
-            inquirer.Text(
-                "contact_name", message='Enter name to save to whitelist (leave blank to skip)'),
+                "workchain", message='Workchain', default="0"),
         ]
         ans = self._prompt(questions)
-
-        name = ans["name"]
+        is_single = ans["single"] == "Single"
         version = WalletVersionEnum(ans["version"])
         workchain = int(ans["workchain"])
-        comment = ans["comment"]
-        contact_name = ans["contact_name"]
 
-        if contact_name:
-            contact = self.ctx.whitelist.get_contact(contact_name)
-            if contact is not None:
-                raise Exception(
-                    f"Contact with the name '{contact_name}' already exists")
+        if is_single:
+            questions = [
+                inquirer.Text("name", message='Wallet name'),
+                inquirer.Text(
+                    "comment", message='Wallet description (leave blank to skip)'),
+            ]
+            ans = self._prompt(questions)
+            name = ans["name"]
+            comment = ans["comment"]
+            questions = [
+                inquirer.Text(
+                    "contact_name", message='Enter name to save to whitelist (blank to skip)', default=name),
+            ]
+            contact_name = self._prompt(questions)["contact_name"]
 
-        mnemonics, pub_k, _priv_k, wallet = Wallets.create(version, workchain)
-        self.ctx.keystore.add_record(name, wallet.address,
-                                     pub_k.hex(), mnemonics, version,
-                                     workchain, comment, save=True)
+            wallets_to_create = [(name, comment, contact_name)]
 
-        if contact_name:
-            self.ctx.whitelist.add_contact(
-                contact_name, wallet.address.to_string(True), save=True)
+        else:
+            questions = [
+                inquirer.Text("number_of_wallets",
+                              message="Number of wallets to create"),
+                inquirer.Text(
+                    "prefix", message='Wallet name prefix (e.g. employee)'),
+                inquirer.Text(
+                    "comment", message='Overall wallets description (leave blank to skip)'),
+                inquirer.Confirm(
+                    "add_all_to_whitelist", message='Add all wallets to whitelist?', default=True),
+            ]
+            ans = self._prompt(questions)
+            number_of_wallets = int(ans["number_of_wallets"])
+            prefix = ans["prefix"] + "_"
+            comment = ans["comment"]
+            add_all_to_whitelist = ans["add_all_to_whitelist"]
 
+            num_of_existing = sum(record.name.startswith(prefix)
+                                  for record in self.ctx.keystore.records)
+            existing_leading_zeros = len(str(num_of_existing))
+            new_leading_zeros = len(str(number_of_wallets + num_of_existing))
+
+            if new_leading_zeros != existing_leading_zeros:
+                zeros_dif = new_leading_zeros - existing_leading_zeros
+                replace_with = "_" + ("0" * zeros_dif)
+                for record in self.ctx.keystore.records:
+                    if record.name.startswith(prefix):
+                        record.name = record.name.replace("_", replace_with)
+                for contact in self.ctx.whitelist.contacts:
+                    if contact.name.startswith(prefix):
+                        contact.name = contact.name.replace("_", replace_with)
+
+            wallets_to_create = []
+            i = 1
+            while len(wallets_to_create) < number_of_wallets:
+                wallet_name_unique = False
+                new_name = None
+                contact_name = ""
+                while not wallet_name_unique:
+                    new_name = f"{prefix}{str(i).zfill(new_leading_zeros)}"
+                    i += 1
+                    if add_all_to_whitelist:
+                        contact_name = new_name
+                        if self.ctx.whitelist.get_contact(contact_name) is not None:
+                            continue
+                    if self.ctx.keystore.get_record_by_name(new_name) is not None:
+                        continue
+                    wallet_name_unique = True
+
+                wallets_to_create.append(
+                    (new_name, comment, contact_name))
+
+        self._start_loading()
+        for wallet_name, comment, contact_name in wallets_to_create:
+            if contact_name:
+                contact = self.ctx.whitelist.get_contact(contact_name)
+                if contact is not None:
+                    raise Exception(
+                        f"Contact with the name '{contact_name}' already exists")
+
+            mnemonics, pub_k, _priv_k, wallet = Wallets.create(
+                version, workchain)
+            self.ctx.keystore.add_record(wallet_name, wallet.address,
+                                         pub_k.hex(), mnemonics, version,
+                                         workchain, comment, save=False)
+
+            if contact_name:
+                self.ctx.whitelist.add_contact(
+                    contact_name, wallet.address.to_string(True), save=False)
+
+        self.ctx.keystore.save()
+        self.ctx.whitelist.save()
+        self._stop_loading()
         echo_success()
 
     def _handle_init_wallet(self):
@@ -156,18 +235,26 @@ class KeystoreSet(BaseSet):
         if wallet_name_to_init == False:
             return
         questions = [
+            inquirer.Confirm(
+                "wait_for_result", message="Wait until transaction will be completed?", default=True),
             inquirer.Password("keystore_password",
                               message='Keystore password'),
         ]
-        keystore_password = self._prompt(questions)["keystore_password"]
+        ans = self._prompt(questions)
+        wait_for_result = ans["wait_for_result"]
+        keystore_password = ans["keystore_password"]
         record = self.ctx.keystore.get_record_by_name(
             wallet_name_to_init, raise_none=True)
         mnemonics = self.ctx.keystore.get_secret(
             record, keystore_password).split(" ")
         _mnemonics, _pub_k, _priv_k, wallet = Wallets.from_mnemonics(mnemonics, record.version,
                                                                      record.workchain)
-        result = self.ctx.ton_client.deploy_wallet(wallet)
-        echo_success(result)
+        result = self.ctx.ton_client.deploy_wallet(wallet, wait_for_result)
+        
+        if wait_for_result:
+            self._parse_broadcast_result(result)
+        else:
+            echo_success()
 
     def _handle_get_wallet(self):
         wallet_name = self.__select_wallet_or_false("Get wallet")
@@ -179,7 +266,7 @@ class KeystoreSet(BaseSet):
 
         questions = [
             inquirer.Confirm(
-                "verbose", message='Show verbose information?', default=True),
+                "verbose", message='Show balances?', default=True),
         ]
         verbose = self._prompt(questions)["verbose"]
 
@@ -226,20 +313,51 @@ class KeystoreSet(BaseSet):
         echo_success()
 
     def _handle_delete_wallet(self):
-        wallet_name = self.__select_wallet_or_false("Delete wallet")
-        if wallet_name == False:
-            return
+        questions = [
+            inquirer.List(
+                "single", message="Choose option", choices=["Single", "Batch"],
+                carousel=True),
+        ]
+        is_single = self._prompt(questions)["single"] == "Single"
+        if is_single:
+            wallet_name = self.__select_wallet_or_false("Delete wallet")
+            if wallet_name == False:
+                return
+            confirm_phrase = f'Are you sure you want to delete {wallet_name} wallet?'
+            names_to_delete = [wallet_name]
+
+        else:
+            questions = [
+                inquirer.Text(
+                    "prefix", message="Etner wallet prefix to delete (e.g. employee)"),
+            ]
+            prefix = self._prompt(questions)["prefix"] + "_"
+            confirm_phrase = f'Are you sure you want to delete all wallets with {prefix} prefix?'
+            names_to_delete = [
+                record.name for record in self.ctx.keystore.records if record.name.startswith(prefix)]
+
         questions = [
             inquirer.Confirm(
-                "is_sure", message=f'Are you sure you want to delete {wallet_name} wallet?', default=False,)
+                "is_sure", message=confirm_phrase, default=False,),
+            inquirer.Confirm(
+                "delete_whitelist", message="Do you want to delete whitelist contacts with the same names?", default=False)
         ]
-        is_sure = self._prompt(questions)["is_sure"]
+        ans = self._prompt(questions)
+        is_sure = ans["is_sure"]
+        delete_whitelist = ans["delete_whitelist"]
 
         if not is_sure:
             echo_success("Action canceled.", True)
             return
 
-        self.ctx.keystore.delete_record(wallet_name, save=True)
+        for name in names_to_delete:
+            self.ctx.keystore.delete_record(name, save=False)
+            if delete_whitelist:
+                self.ctx.whitelist.delete_contact(name, save=False)
+
+        self.ctx.keystore.save()
+        if delete_whitelist:
+            self.ctx.whitelist.save()
 
         echo_success()
 
@@ -264,24 +382,26 @@ class KeystoreSet(BaseSet):
             inquirer.Text(
                 "mnemonics", message="Mnemonic words (splited by space)"),
             inquirer.List(
-                "version", message='Wallet version', choices=[e.value for e in WalletVersionEnum], carousel=True),
+                "version", message='Wallet version', choices=[e.value for e in WalletVersionEnum],
+                carousel=True, default=WalletVersionEnum.v4r2),
             inquirer.Text(
-                "workchain", message='Workchain'),
+                "workchain", message='Workchain', default="0"),
+            inquirer.Text("name", message='Wallet name'),
         ]
         ans = self._prompt(questions)
         mnemonics = ans["mnemonics"].split(" ")
         version = WalletVersionEnum(ans["version"])
         workchain = int(ans["workchain"])
+        name = ans["name"]
 
         mnemonics, pub_k, _priv_k, wallet = Wallets.from_mnemonics(
             mnemonics, version, workchain)
 
         questions = [
-            inquirer.Text("name", message='Wallet name'),
             inquirer.Text(
                 "comment", message='Wallet description (leave blank to skip)'),
             inquirer.Text(
-                "contact_name", message='Enter name to save to whitelist (leave blank to skip)'),
+                "contact_name", message='Enter name to save to whitelist (blank to skip)', default=name),
         ]
         ans = self._prompt(questions)
         name = ans["name"]
